#!/bin/bash 


# -  Library:       WebSphere MQ
# -  Component:     WMQ SOAP Sample Programs
# -  Part:          SecondAxis.java
# - 
# -   <START_COPYRIGHT> 
# -   Licensed Materials - Property of IBM 
# -                    
# -   5724-B41         
# -   (C) Copyright IBM Corp. 1994, 2003 All Rights Reserved. 
# -                    
# -   US Government Users Restricted Rights - Use, duplication or 
# -   disclosure restricted by GSA ADP Schedule Contract with 
# -   IBM Corp.        
# -   <END_COPYRIGHT>

# - this prepares a JAVA file for access by SOAP/WMQ
# - see help (at end, called by deployWMQService with no arguments) for details.

todo=default


# ======================================================================================

setup()
   {
   # - standard classpath etc for all MQSOAP stuff
   my_dir="`cd \`dirname $0\` 2>/dev/null && pwd`"
   . ${my_dir}/setcp.sh
   
   # - set default queue manager unless overridden
   mqqm=""
   mqq=#default#

   while [ $# -ne 0 ]
      do
      case $1 in
   
         "?"|"-?"  )
            usage
            shift
         ;;

      # - pick up queue manager if defined
      "-m" )
         mqqm=$2
         shift
         shift
      ;;

      # -pick up queue if defined
      "-q" )
         mqq=$2
         shift
         shift
      ;;

      # - pick up command todo
      "-c" )
         todo=$2
         shift
         shift
      ;;

      # - pick up command todo
      "-echo" )
         set -x
         shift
         shift
      ;;

      * ) className=$1
          shift
      ;;

      esac
      done


   # ------------------

   # - remove the .java if specified, and make sure in class (.) not directory (/) format
   if [ "`filext $className`" == "java" ]
   then
      className=`strip_suffix $className`
      if [ "$todo" == "default" ]
      then
         todo=allAxis
      fi
   fi

   if [ "`filext $className`" == "asmx" ]
   then
      className=`strip_suffix $className`
      if [ "$todo" == "default" ]
      then
         todo=allAsmx
      fi
   fi

   #set className=%className:\=.%
   className=`class_format $className`

   #set classSourceDir=%className:.=\%
   classSourceDir=`dir_format $className`

   # - equal if no package used, in which case Axis will generate package DefaultNamespace for proxies
   proxyDir=$classSourceDir
   if [ "$classSourceDir" == "$className" ]
   then
      proxyDir=DefaultNamespace/$className
   fi
   proxyName=`class_format $proxyDir`

   # - prepare some useful derived names
   wmq_wsdl=${className}_Wmq.wsdl
   if [ "$mqq" == "#default#" ]
   then
      mqq=SOAP.$className
   fi

   if [ "$mqqm" == "" ]
   then
      wmqUri=wmq:$mqq
   else
      wmqUri=wmq:$mqq@$mqqm?connectQueueManager=$mqqm,replyToQueue=MQSOAP.RESPONSE.QUEUE
   fi

   servicePort=${className}_Wmq

   # - create work directories (if needed)
   if [ ! -d helpers ]
   then
      mkdir helpers 
   fi

   return 0
}

# --------------- end setup

# ======================================================================================

allAxis()
   {
   compileJava
   if [ $? -ne 0 ]
   then
      return 1
   fi

   genAxisWsdl
   if [ $? -ne 0 ]
   then
      return 1
   fi

   axisDeploy
   if [ $? -ne 0 ]
   then
      return 1
   fi

   genAxisWMQBits
   if [ $? -ne 0 ]
   then
      return 1
   fi

   genProxiesToAxis
   if [ $? -ne 0 ]
   then
      return 1
   fi

   return 0
   }
# --------------- end allAxis

# ======================================================================================

allAsmx()
   {
   genAsmxWsdl
   if [ $? -ne 0 ]
   then
      return 1
   fi

   genAsmxWMQBits
   if [ $? -ne 0 ]
   then
      return 1
   fi

   #genProxiesToDotNet
   #if [ $? -ne 0 ]
   #then
   #   return 1
   #fi

   return 0
   }
# --------------- end allAxis

# ======================================================================================
compileJava()
   {
   # - compile the java, create target directory if necessary
   if [ ! -d classes ]
   then
      mkdir classes
   fi

   javac -d classes ${classSourceDir}.java
   retcode=$?
   if [ $retcode -ne 0 ]
   then
      echo Failure in compilation of ${proxyDir}.java
      exit $retcode
   fi
   return 0
   }
# ------------ end compile

# ======================================================================================
genAxisWsdl()
   {
   # - generate the correct wsdl, do not override any old versions
   if [ -f helpers/${wmq_wsdl} ]
   then
      wsdlin="--input helpers/${wmq_wsdl}"
   else
      wsdlin=""
   fi

   java org.apache.axis.wsdl.Java2WSDL $wsdlin \
            --output helpers/${wmq_wsdl} \
            --location "${wmqUri}" \
            --bindingName ${className}BindingSoap \
            --servicePortName $servicePort $className 

   if [ ! -f helpers/${wmq_wsdl} ]
   then
      echo Failure in building wsdl
      return 5
   fi
   return 0
   }
# --------------- end genAxisWsdl

# ======================================================================================
genAsmxWsdl()
   {
   my_dir="`cd \`dirname $0\`/.. 2>/dev/null && pwd`"

   ${my_dir}/mqsoapwsdl $wmqUri ${className}.asmx helpers/${wmq_wsdl}
   retcode=$?
   if [ ! -f helpers/${wmq_wsdl} ]
   then
      echo Failure in building Wsdl
      return 5
   fi

   if [ $retcode -ne 0 ]
   then
      echo Error in building Wsdl, see helpers/${wmq_wsdl} for details
      exit $retcode
   fi

   return 0
   }
# --------------- end genAxisWsdl

# ======================================================================================

genAxisWMQBits()
   {
   my_dir="`cd \`dirname $0\` 2>/dev/null && pwd`"
   # - prepare a tailored listener with helper .bat file (and queues to match)
   listen="java com.ibm.axis.transport.wmq.SimpleJMSListener -u "$wmqUri" -s $servicePort"
   listenbat=helpers/listen_${className}.sh  
   echo "#!/bin/bash" >> $listenbat
   echo "" >> $listenbat
   echo "# - generated by $0  at `date`" >> $listenbat
   echo "export MA0R_HOME=${MA0R_HOME}" >> $listenbat
   echo ". ${my_dir}/setcp.sh" >> $listenbat
   echo "cd `pwd`" >> $listenbat
   echo "$listen" >> $listenbat
   chmod ugo+x $listenbat

   # - define queue for MQ, and set it up to trigger its listener
   runMQCommand "DEFINE QL('SOAP.INITQ') REPLACE"
   runMQCommand "DEFINE QL('${mqq}') TRIGTYPE(FIRST) TRIGGER PROCESS('${className}.PROCESS') INITQ('SOAP.INITQ') TRIGMPRI(0) REPLACE"
   runMQCommand "DEFINE PROCESS('${className}.PROCESS') APPLICID('xterm -T \"JMS WMQSoapListener - ${className}\" -e `pwd`/${listenbat}') REPLACE"

   return 0
   }
# --------------- end genAxisWMQBits

# ======================================================================================


startWMQListener()
   {
   xterm -T "MQ SOAP trigger monitor" -e runmqtrm -m $mqqm -q SOAP.INITQ &
   return 0
   }
# --------------- end genAxisWMQBits
# ======================================================================================


genProxiesToAxis()
   {
   # - Generate all proxies (C#, VB and Java) and server wsdd file, and compile the Java
   pushd helpers > /dev/null
   #quietOK "${msfwdir}/wsdl" $wmq_wsdl  
   #quietOK "${msfwdir}/wsdl" $wmq_wsdl /language:VB

   java com.ibm.mq.ma0r.tools.RunWSDL2Java $wmq_wsdl
   if [ ! -d classes ]
   then
      mkdir classes 
   fi

   tempdir=`dirname $proxyDir`
   javac -d classes ${tempdir}/*.java
   popd > /dev/null
   return 0
   }
# --------------- end genProxies

# ======================================================================================
axisDeploy()
   {
   # - Generate the server side wsdd file
   # - There is not an option in to avoid generating the client stuff
   # - so we work in a temporary directory, copy out what we want, and then erase the directory.
   # -
   # - Where the client stuff is required as well, calling twice for the proxies and the wsdd is waseful
   # - but improves the code modularity.
   # -
   # - java com.ibm.mq.ma0r.tools.RunWSDL2Java is the same as org.apache.axis.wsdl.WSDL2Java,
   # - except that it allows for the wmq: URL syntax.
   pushd helpers > /dev/null
   echo  
   quietOK java com.ibm.mq.ma0r.tools.RunWSDL2Java --server-side --output temp.server $wmq_wsdl
   # - patch the wsdd -- Axis does not seem to understand that the wsdl was generated bottom up.
   # - This very primitive substituation points the wsdd directly at our original class as implementation,
   # - rather than at the newly generated implementation skeleton.
   # -
   # - make sure target directory ready
   tempDir=`dirname $classSourceDir`
   if [ ! -d $tempDir ]
   then
      mkdir -p $tempDir 
   fi

   # - find the temporary source directory
   tempDir=`dirname ./temp.server/${proxyDir}`

   # - now do the patching
   java com.ibm.mq.ma0r.util.PatchWsdd $className < ${tempDir}/deploy.wsdd > ${classSourceDir}_deploy.wsdd
   java com.ibm.mq.ma0r.util.PatchWsdd $className < ${tempDir}/undeploy.wsdd > ${classSourceDir}_undeploy.wsdd

   rm -rf temp.server
   popd > /dev/null

   # - deploy using the generated (and modified) wsdd file
   java org.apache.axis.utils.Admin server helpers/${classSourceDir}_deploy.wsdd

   return 0
   }
# --------------- end axisDeploy
# ======================================================================================


# ======================================================================================
# - helper functions
# - given filename with path, extract just the directory, and set into variable tempDir
getDir()
   {
   tempDir=`dirname $1`
   return 0
   }

# - execute a command quietly (unless it fails)
quietOK()
   {
   $* >quietOK.templog  2>quietOK.temperrlog
   quietOKExit=$?
   if [ $quietOKExit -ne 0 ] 
   then
      echo "+++++++++++++++ quietOK $* gave errlevel = $quietOKExit"
      cat quietOK.templog
      cat quietOK.temperrlog
   fi

   rm -f quietOK.templog
   rm -f quietOK.temperrlog

   return $quietOKExit
   }

# ======================================================================================
runMQCommand()
   {
   # - execute a WMQ command, quitely if it works
   echo $* | runmqsc $mqqm  1>quietOK.templog  2>quietOK.temperrlog
   quietOKExit=$?
   if [ $quietOKExit -ne 0 ] 
   then
      echo +++++++++++++++ runmqsc gave errlevel = $quietOKExit
      type quietOK.templog
      type quietOK.temperrlog
   fi

   rm -f quietOK.templog
   rm -f  quietOK.temperrlog
   return $quietOKExit
   }

# ======================================================================================

# - help
errhelp()
{
echo Incorrect parameters given to $0 +++++++++++++++++++++++++++++++++++++++++++++++++++++++
echo -
usage
}

usage()
{
echo "$0 [options] className:        prepare classname for access by SOAP/WMQ"
echo "-"
echo "arguments:"
echo "    className:       name of Java class or file to be deployed (.java optional)."
echo "                       java file must be in the conventional package dependent subdirectory"
echo "options:"
echo "    -echo ON/OFF     set trace on or off for execution of this file"
echo "    -m qmgrName      queue manager to use (default is the default queue manager)"
echo "    -q queueName     queue to use (default is \"SOAP.<className>\")"
echo "    -c command       command to execute, where command is:"
echo "        allAxis         peform all compile and setup steps for an Axis service"
echo "           compileJava         compile the Java service (.java   to .class)"
echo "           genAxisWsdl         generate Wsdl            (.class  to .wsdl)"
echo "           axisDeploy          deploy the class file    (.wsdl   to .wsdd, apply .wsdd)"
echo "           genProxiestoAxis    generate proxies         (.wsdl   to .java, .class, .cs and .vb)"
echo "           genAxisWMQBits      setup WMQ queues, listeners and triggers"
echo "-"
echo "        allAsmx         peform all setup steps for an Asmx service"
echo "           genAsmxWsdl         generate Wsdl            (.asmx   to .wsdl)"
echo "           genProxiesToDotNet  generate proxies         (.wsdl   to .java, .class, .cs and .vb)"
echo "           genAsmxWMQBits      setup WMQ queues, listeners and triggers"
echo "-"
echo "outputs:"
echo "    The source file is compiled into \"./classes\""
echo "    Other files are created in \"./helpers\"",
echo "      and various package dependent \"./helpers/<package>\" subdirectories."
echo "      Java proxies are compiled into \"./helpers.classes\"."
echo "-"

exit 1
}


filext()
   {
   if [ "$1" != "" ]
   then
      echo `basename $1` | rev | cut -d. -f1 | rev
   fi
   }

class_format()
   {
   echo $1 | sed 's#/#\.#g'
   }

dir_format()
   {
   echo $1 | sed 's#\.#/#g'
   }

strip_suffix()
   {
   if [ "$1" != "" ]
   then
      echo $1 | rev | cut -d. -f2 | rev
   fi
   }

main()
   {
   setup $*
   if [ $? -ne 0 ]
   then
      exit 1
   fi

   $todo
   if [ $? -ne 0 ]
   then
      exit 10
   fi

   exit 0
   }


main $* 





