#!/bin/bash 

# -  Library:       WebSphere MQ
# -  Component:     WMQ SOAP Sample Programs
# -  Part:          SecondAxis.java
# - 
# -   <START_COPYRIGHT> 
# -   Licensed Materials - Property of IBM 
# -                    
# -   5724-B41         
# -   (C) Copyright IBM Corp. 1994, 2003 All Rights Reserved. 
# -                    
# -   US Government Users Restricted Rights - Use, duplication or 
# -   disclosure restricted by GSA ADP Schedule Contract with 
# -   IBM Corp.        
# -   <END_COPYRIGHT>

QMGR=MQSOAP.DEMO.QM

do_clearq()
   {
   echo "Setting up required MQ queues..."
   runmqsc ${QMGR} 2>&1 > /dev/null << EOF

   define  ql('MQSOAP.REQUEST.QUEUE')
   define  ql('MQSOAP.RESPONSE.QUEUE')
   define  ql('SOAP.javademos.server.StockQuoteAxis')
   define  ql('SOAP.INITQ')

   clear  ql('MQSOAP.REQUEST.QUEUE')
   clear  ql('MQSOAP.RESPONSE.QUEUE')
   clear  ql('SOAP.javademos.server.StockQuoteAxis')
   clear  ql('SOAP.INITQ')

   end
EOF
   }

do_qmgr()
   {
   echo "Checking Queue Manager ${QMGR} is available..."
   runmqsc ${QMGR} 2>&1 > /dev/null << EOF
   display qmgr
   end
EOF

   if [ $? -ne 0 ] 
   then
      echo "   Queue manager not available, attempting to start..."
      strmqm ${QMGR} 
      if [ $? -ne 0 ] 
      then    
         echo "   Could not start queue manager, attempting to create..."
         crtmqm ${QMGR}
         if [ $? -ne 0 ]
         then
            echo "   ERROR - could not create queue manager, aborting."
            exit -1
         else
            echo "Starting new queue manager..."
            strmqm ${QMGR}
            if [ $? -ne 0 ]
            then
               echo "   ERROR - could not start newly created queue manager, aborting."
               exit -1
            fi
         fi
      fi      
   fi
   }
 

do_runmqlsr()
   {
   echo "Checking runmqlsr..."
   if [ 0 -eq `ps -efw | grep runmqlsr | grep -v grep | wc -l` ]
   then
      echo "   runmqlsr process is not running, attempting to start..."
      ( runmqlsr -t TCP -m ${QMGR} 2>&1 | tee /tmp/runmqlsr.log ) > /dev/null &
      if [ $? -ne 0 ]
      then
         echo "   ERROR - could not start runmqlsr process, aborting..."
         exit -1
      fi
   fi
   }


main()
   {
   do_qmgr
   do_clearq
   # do_runmqlsr
   }

main 


