#!/bin/bash
# - demonstration of deploying and using new classes

pause()
   {
   if [ "$1" != "nopause" ]
   then
      echo "Please hit a key to continue..."
      read
   fi
   }

echo "-"
echo "------------------------------------------------------------------------"
echo "-"
echo "- $0 [nopause]"
echo "-"
echo "This sample walks through the steps of creating, deploying and testing a new Java service"
echo "to be called using the WebSphere MQ Transport for SOAP."
echo "-"
echo "Unless nopause is specified, it will pause at each step."
echo "-"
echo "This uses a 'bottom up' technique, starting with the Java service,"
echo "and moving up through wsdl and proxy generation to coding a client."
echo "-"
echo "Steps are:"
echo "  Step 0: Clean up any old deployments (more for testing)."
echo "  Step 1: Code the service (prepared in advance)."
echo "  Step 2: Compile the service and prepare it for use."
echo "  Step 3: Code the clients  (prepared in advance)."
echo "  Step 4: Compile the clients."
echo "  Step 5: Test everything out."
echo "-"
echo "------------------------------------------------------------------------"
echo "-"
echo "Step 0: clean up any old deployments"
rm -rf helpers
rm -rf classes
rm -f service-config.wsdd
echo "-"
pause $*
echo "------------------------------------------------------------------------"
echo "-"
echo "Step 1: code the service"
echo "   We have prepared step1 in advance, see StockQuoteAxis.java and SecondAxis.java"
ls -1 *.java
echo "-"
pause $*
echo "-"
echo "------------------------------------------------------------------------"
echo "-"
echo "Step 2: compile the service and prepare it for use."
echo "   This invokes deployWMQService.bat, which in turn does many things, as ..."
../../bin/deployWMQService.sh -?
echo "-"
echo "   We will now call deployWMQService for each Java service."
echo "-"
pause $*
echo "   First call with no package, Axis will use DefaultNamespace for all proxies etc"
../../bin/deployWMQService.sh -m MQSOAP.DEMO.QM StockQuoteAxis.java 
echo "-"
echo "   Second call with explicit package, use class notation so we need to warn that it is Axis"
../../bin/deployWMQService.sh -m MQSOAP.DEMO.QM -c allAxis secondPackage.SecondAxis 
echo "-"
echo "Deployment and preparation now complete."
pause $*
echo "-"
echo "------------------------------------------------------------------------"
echo "-"
echo "Step 3: code the clients"
echo "   We have prepared step3 in advance, in the testClient subdirectory"
ls -1 testClient
echo "-"
pause $*
echo "-"
echo "------------------------------------------------------------------------"
echo "-"
echo "Step 4: compile the clients."
echo "  buildClients.bat demonstrates this."
echo "  Note that buildClients picks up the appropriate generated proxies "
echo "  from the preparation step2."
pushd testClient > /dev/null
./buildClients.sh
popd > /dev/null
echo "-"
echo "Clients now built."
echo "Deployment complete, now ready to start execution"
echo "-"
pause $*
echo "-"
echo "------------------------------------------------------------------------"
echo "-"
echo "Step 5: test everything out"
echo "   We will start the required listeners/servers and then run"
echo "   one of the test clients."
echo "-"
echo "-"
pause $*
echo "-"
echo "   Start the required listeners."
# Commented out pending resoultion of difficulties triggering to Java programs
#../../bin/deployWMQService.sh -m MQSOAP.DEMO.QM -c startWMQListener secondPackage.SecondAxis 
xterm -T StockQuoteAxis -e ./helpers/listen_StockQuoteAxis.sh &
xterm -T secondPackage.secondAxis -e ./helpers/listen_secondPackage.SecondAxis.sh &
echo "-"
echo  "   Start a Java client"
./testClient/StockQuoteAxisClientJava.sh
echo "-"
echo "------------------------------------------------------------------------"
echo "-"
echo "   Deployment and Test now complete"
echo "-"
echo "------------------------------------------------------------------------"
echo "-"
pause $*
exit 0 




