// Library:       WebSphere MQ
// Component:     WMQ SOAP Sample Programs
// Part:          RunIvt.java
//
//  <START_COPYRIGHT> 
//  Licensed Materials - Property of IBM 
//                   
//  5724-B41         
//  (C) Copyright IBM Corp. 1994, 2003 All Rights Reserved. 
//                   
//  US Government Users Restricted Rights - Use, duplication or 
//  disclosure restricted by GSA ADP Schedule Contract with 
//  IBM Corp.        
//  <END_COPYRIGHT>

package javaDemos.clients;

import java.io.*;
import java.util.*;

import com.ibm.mq.ma0r.util.GlobalLog;
import com.ibm.mq.ma0r.util.Killme;
// not yet supported  import java.util.regex.Pattern;

public class RunIvt {

  private static String START = "startwin.bat";
  private static final String CONFIGFILE = "ivttests.txt";
  private static final String LINUXCONFIGFILE = "ivttests_linux.txt";
  private static final String JAVA_CMD = "java";

  FileReader file;
  BufferedReader buff;
  private String cmd;       // cmd to run for test
  private String task;      // name of test
  private String response;  // expected response
  private String desc;      // description
  private String requires;  // list of prereq servers for test
  private boolean usingHttpax=false;
  private boolean usingJMSax=false;
  private boolean usingDotNet=false;
  private boolean hold = false;  // true to prevent killing servers
  private boolean killall = false; // true to kill ALL servers
  private String configFile=null;

//  private boolean usingTcpmon=false;
  Process tcpmonp = null;

  int testsRan = 0;
  int testsOK = 0;

  RunIvt(String argv[])
  {
    System.out.println("\n\n");

    if (onWindows())
      START="startwin.bat";
    else
      START="./startwin.sh";

    /*sjpt: only for HTTPR
    try {
      new File("stateFile").delete();
      new File("stateFile.lock").delete();
    } catch (Exception e) {
    }
    */

    SetupMQ();

    // Process argument list

    int numArgs = argv.length;
    for (int i=0; i < argv.length; i++)
    {
      if (argv[i].equalsIgnoreCase("killall"))
      {
        killall=true;
        numArgs--;
      }

      if (argv[i].equalsIgnoreCase("hold"))
      {
        hold=true;
        numArgs--;
      }

      // Look for -c <config-file>
      if (argv[i].equals("-c"))
      {
        if (argv.length > i+1)
        {
          configFile = argv[++i];  
          System.out.println("Using test configuration file: " + configFile);
          numArgs--;
        }
        numArgs--;
      }
    }

    //Run the tests

    OpenConfig();

    // For all tests in the config file
    while (GetaTest())
    {
      // Do we run this test?
      if (numArgs == 0 || testSpecified(argv, task))
      {
        head(task, desc);
        startRequired();
        testsRan++;
        runTest(cmd, response);
      }
    }
    CloseConfig();

    System.out.println("----------------------------------\n");
    stopRequired();
    CloseMQ(); 
    System.out.println("===========================================");
    System.out.println(testsRan + " tests run, of which " + (testsRan-testsOK) + " failed.\n\n");
  }

  private boolean testSpecified(String argv[], String aTest)
  {
    for (int i=0; i< argv.length; i++)
    {
      if (aTest.equalsIgnoreCase(argv[i])) return true;
    }
    return false;
  }

  private void startRequired()
  {
    requires = requires.toLowerCase();
    if (0 <= requires.indexOf("httpax")) useHttpax();
    if (0 <= requires.indexOf("jmsax")) useJMSax();
    //if (0 <= requires.indexOf("tcpmon")) useTcpmon();
    if (0 <= requires.indexOf("dotnet")) useDotNet();
  }

  private void stopRequired()
  {
    if (hold) return;
    //killTcpmon();
    killJMSax();
    killHttpax();
    killDotNet();
  }

  private void OpenConfig()
  {
    try
    {
      if (configFile == null)
      {
        if (onWindows())
          configFile = CONFIGFILE;
        else
          configFile = LINUXCONFIGFILE;
      }

      file = new FileReader(configFile);

      buff = new BufferedReader(file);
    } catch (IOException e)
    {
      System.out.println("ERROR: " + e.toString());
    }
  }

  private void CloseConfig()
  {
    try
    {
      buff.close();
    } catch (IOException e)
    {
      System.out.println("ERROR: " + e.toString());
    }
  }

  private String GetConfigLine()
  {
    try
    {
      boolean eof=false;
      while (!eof)
      {
        String line = buff.readLine();
        if (line == null)
          eof = true;
        else
        {
          line.trim();
          if (0 == line.length()) continue;
          if (line.startsWith("#")) continue;
          return line;
        }
      }
    } catch (IOException e)
    {
      System.out.println("ERROR: " + e.toString());
    }

    return null;
  }

  private boolean GetaTest()
  {
    if (null == (task     = GetConfigLine())) return false;
    if (null == (desc     = GetConfigLine())) return false;
    if (null == (cmd      = GetConfigLine())) return false;
    if (null == (response = GetConfigLine())) return false;
    if (null == (requires = GetConfigLine())) return false;
    return true;
  }

  private void head(String title, String desc)
  {
    System.out.println("\n----- [" + title + "] --------------------------------");
    System.out.println(desc);
  }

  private void SetupMQ()
  {
    if (onWindows())
      CmdExec("setupMQ.bat");
    else
      CmdExec("./setupMQ.sh");
  }

  private void CloseMQ()
  {
    if (onWindows())
      CmdExec("endMQ.bat");
    else
      CmdExec("./endMQ.sh");
  }

  private void useHttpax()
  {
    if (!usingHttpax)
    { 
      if (!Killme.ruThere("SimpleAxisServerX", "localhost"))
      {
        // sjpt StartWindow(JAVA_CMD + " com.ibm.axis.transport.http.SimpleAxisServerX -p 8088",
        if (onWindows())
          StartWindow("SimpleAxisServerX", 
                      "SimpleAxisServerX");
        else
          StartWindow("./simpleAxisServerX.sh", 
                      "SimpleAxisServerX");
      } else
        System.out.println("SimpleAxisServerX already running.");

      usingHttpax=true;
    }
  }

  private void killHttpax()
  {
    if (usingHttpax || killall)
    {
      Killme.kill("SimpleAxisServerX", "localhost"); 
      usingHttpax=false;
    }
  }

  private void useJMSax()
  {
    if (!usingJMSax)
    {
      if (!Killme.ruThere("SimpleJMSListener", "localhost"))
      {
        //sjpt  StartWindow(JAVA_CMD + " com.ibm.axis.transport.wmq.SimpleJMSListener -a",
        if (onWindows())
          StartWindow("helpers\\listen_javaDemos.server.StockQuoteAxis",
                      "SimpleJMSListener");
        else
          StartWindow("./helpers/listen_javaDemos.server.StockQuoteAxis.sh",
                      "SimpleJMSListener");
      } else
        System.out.println("SimpleJMSListener already running.");

      usingJMSax=true;
    }
  }

  private void killJMSax()
  {
    if (usingJMSax || killall)
    {
      Killme.kill("SimpleJMSListener", "localhost"); 
      usingJMSax=false;
    }
  }

  private void useDotNet()
  {
    if (!usingDotNet)
    {
      if (!Killme.ruThere("DotNet", "localhost"))
      {
//sjpt         //System.out.println("Executing: " + REGISTERDOTNET);
//sjpt now done by setcp         CmdExec(REGISTERDOTNET);

        /* sjpt 
        // Retrieve the current directory (which should be the demos sub-directory)
         File dir = new File(".");
         String curDir="";  
  
         try 
            {
            curDir = dir.getCanonicalPath();
            }
         catch(Exception e) 
            {
            e.printStackTrace();
            }
         String STARTSERVER="MQSoapHost MQSOAP.DEMO.QM DOTNETSERVICE.REQUEST.QUEUE " + curDir + " /dummy StockQuoteService.asmx";
         */   

        String STARTSERVER="helpers\\listen_StockQuoteDotNet";

        //System.out.println("Starting: " + STARTSERVER);
        StartWindow(STARTSERVER, "MQSoapListener"); 
      } else
        System.out.println("DotNet listener MQSoapHost already running.");

      usingDotNet=true;
    }
  }

  private void killDotNet()
  {
    if (usingDotNet || killall)
    {
      Killme.kill("DotNet", "localhost"); 
      usingDotNet=false;
    }
  }



  /* private void useTcpmon()
  {
    if (tcpmonp == null)
    {
      System.out.println("\n===> You may see some 'connection refused' messages while tcpmon gets started. <===\n");

      try {
        tcpmonp = Runtime.getRuntime().exec("java org.apache.axis.utils.tcpmon 6666 localhost 8089");
      } catch (IOException e) {
        GlobalLog.logErrorTrace("Cannot start tcpmon", e);
      }
    }
  } */

  /* private void killTcpmon()
  {
    if (tcpmonp != null)
    {
      tcpmonp.destroy();
    }
  } */


  private void StartWindow(String command, String winTitle)
  {
    System.out.println("+++ server: " + command);
    CmdExec(START + " " + winTitle + " " + command);
  }

  private void runTest(String aDemo, String expected)
  {
    String command = aDemo;
    System.out.println("--- client: " + command);
    System.out.println("");
    String actual = CmdExec(command);
// not yet supported      if (!Pattern.matches(expected, mactual))
    if (! actual.endsWith(expected)) {
      System.out.println("\n************** TEST FAILED !! TEST FAILED !! TEST FAILED *************");
      System.out.println(  "EXPECTED: " + expected);
      System.out.println(  "ACTUAL:   " + actual);
      System.out.println(  "**********************************************************************\n");
    } else {
      System.out.println("OK.");
      testsOK++;
    }
  }

  private boolean StillRunning(Process p)
  {
    try
    {
      int x = p.exitValue();
      return false;
    } catch (Exception err)
    {
      return true;
    }
  }

  /* private String CmdExec(String cmdline) {
    String line, lastline;
    lastline="";
    try {
      Process p = Runtime.getRuntime().exec(cmdline);

      BufferedReader input = 
        new BufferedReader
        (new InputStreamReader(p.getInputStream())); 

      while ((line = input.readLine()) != null) {
        if (line.startsWith("Exiting")) break;
        if (line.startsWith(">>> EXCEPTION")) break;
        if (line.startsWith("LOG: Attempting download of new URL file:")) break;
        System.out.println(line);
        lastline=line;
      } 

      input.close();
      // p.waitFor();
    } catch (Exception err) {
      System.out.println("Failed to start process: " + cmdline);
      err.printStackTrace();
    }

    return lastline;
  } */

  private String CmdExec(String cmdline) {
    String line, lastline;
    lastline="";
    boolean done=false;

    try {
      Runtime myrun = Runtime.getRuntime();
      Process p = myrun.exec(cmdline);

      BufferedReader input = 
        new BufferedReader
        (new InputStreamReader(p.getInputStream())); 

      BufferedReader error = 
        new BufferedReader
        (new InputStreamReader(p.getErrorStream())); 

      while (!done && StillRunning(p))
      {
        Thread.sleep(50);
        while (!done && (input.ready() || error.ready()))
        {
          if (input.ready())
          {
            line = input.readLine();
            if (line.startsWith("Exiting")) done=true;
            if (line.startsWith(">>> EXCEPTION")) done=true;
            if (line.startsWith("LOG: Attempting download of new URL file:")) done=true;
            System.out.println(line);
            if (!done) lastline=line;
          }

          if (error.ready())
          {
            line = error.readLine();
            if (line.startsWith("Exception in thread"))
            {
              lastline=line;
              done=true;
            }
          }
        }
      }

      input.close();
      error.close();

    } catch (Exception err) {
      System.out.println("Failed to execute process: " + cmdline);
      err.printStackTrace();
    }

    return lastline;
  }

  private boolean onWindows2K()
  {
    if (System.getProperty("os.name").equals("Windows 2000"))
      return true;
    return false;
  }

  private boolean onWindows()
  {
    if (System.getProperty("os.name").toLowerCase().startsWith("win"))
      return true;
    return false;
  }

  public static void main(String argv[]) {
    new RunIvt(argv);
  }
}